from artifact_ddd.utils import get_dateref_yyyymm
import pandas as pd
import pytest

def test_get_dateref_yyyymm_debito(mock_dataframe):
    df = pd.DataFrame(mock_dataframe)
    result = get_dateref_yyyymm(df, "data", "%d/%m/%Y","debito")
    assert result == "202504"

def test_get_dateref_yyyymm_credito(mock_dataframe):
    df = pd.DataFrame(mock_dataframe)
    result = get_dateref_yyyymm(df, "data", "%d/%m/%Y","credito")
    assert result == "202505"

def test_except_ingest_expenses_sor(mock_dataframe):
    with pytest.raises(Exception) as error:
        result = get_dateref_yyyymm(mock_dataframe,"data", "%d/%m/%Y","debito")    
    assert ('Erro ao capturar o dataref da tabela' in str(error.value))


__version__ = "0.0.1"

from .datamesh import ingest_expenses_sor  # ajuste para o nome real que quiser expor
from .utils import get_dateref_yyyymm

__all__ = [
    "ingest_expenses_sor",
    "get_dateref_yyyymm"
]


from dateutil.relativedelta import relativedelta
import pandas as pd


def get_dateref_yyyymm(df: pd.DataFrame, date_column:str,
                        format_field:str, sell_type: str)-> str:
    """
    Identifica a data referência em uma coluna de um DataFrame pandas e retorna
    o ano e o mês no formato YYYYMM.

    :param df: DataFrame contendo a coluna de datas.
    :param date_column: Nome da coluna que contém as datas.
    :param format_field: formato da data que o campo possuí. 
    :param sell_type: tipo de despesa do arquivo.
    
    :return str: ano mês min/max no formato YYYYMM.
    """
    try:
        # Converte a coluna para datetime, se ainda não for
        df[date_column] = pd.to_datetime(
                            df[date_column], 
                            format=format_field,
                            errors='coerce')
        
        # Encontra a data máxima e retorna no formato desejado
        if sell_type == 'debito':
            dateref = df[date_column].min()

        if sell_type == 'credito':
            dateref = df[date_column].max()
            dateref = dateref + relativedelta(months=1)

        return dateref.strftime('%Y%m') if pd.notnull(dateref) else None
    
    except Exception as e:
            raise Exception('[utils - functions.get_dateref_yyyymm] Erro ao '
                f'capturar o dataref da tabela: (Tipo:{sell_type}, erro:{e})')



from artifact_ddd.datamesh import ingest_expenses_sor
from unittest.mock import patch, MagicMock
import pandas as pd
import logging
import pytest
import time


logger = logging.getLogger(__name__)
logger.setLevel(logging.INFO)

@patch('artifact_ddd.datamesh.wr', MagicMock())
def test_ingest_expenses_sor(mock_dataframe):
    df = pd.DataFrame(mock_dataframe)
    version = int(time.time())
    df['numero_versao_processamento'] = version
    user = "thorfin"

    result = ingest_expenses_sor(df, user)
    logger.info(f'teste efetuado! Versão:{version}')
    assert result == {"linhas_gravadas": len(df)}

@patch('artifact_ddd.datamesh.wr', MagicMock())
def test_except_ingest_expenses_sor(mock_dataframe):
    user = "thorfin"
    version = int(time.time())
    with pytest.raises(Exception) as error:
        result = ingest_expenses_sor(mock_dataframe, user)
    
    assert ('[datamesh.ingest_expenses_sor] erro ao inserir dados' in
             str(error.value))


import awswrangler as wr
import pandas as pd 

def ingest_expenses_sor(df: pd.DataFrame, user: str) -> dict:
    """
    Função responsável por preencher a tabela despesas_sor de acordo com o 
    banco de dados que o usuário está executando

    :param df: dataframe com os gastos que serão inseridos na tabela.
    :param user: usuário que está executando a aplicação.    
    :return: dicionário com a quantidade de linhas inseridas na tabela.
    """
    try:
        database = 'db_source_expenses_' + user
        table = 'despesas_sor' 
        for field in df.columns:
            df[field] = df[field].astype(str).replace({'nan': None, 'NaT':None, 
                'None': None}) 

        # Escrever (append)
        wr.s3.to_parquet(
            df=df, 
            path=f's3://ddd-dbsource-datamesh/{database}/{table}',
            dataset=True,
            database=database,
            table=table,
            mode="append"
        ) 

        return {"linhas_gravadas": len(df)}
    
    except Exception as e:
        raise Exception('[datamesh.ingest_expenses_sor] erro ao inserir dados'
            f' na tabela:{e}')
